// Some external parameters
#include "colors.inc"  
#include "metals.inc"  
#include "woods.inc"  
#include "glass.inc"

// Parameters for the picture
#declare Length = 80;

///////////////////
// Some Useful Objects
///////////////////

// Some dimensions
#declare Thick = 0.10; // The wooden sticks
#declare Thin = 0.05;  // The needles
#declare VThin = 0.02; // The helper lines

// Make a plane that contains three points
// These are helper planes so we give a glass finish
#macro Plane(First,Second,Third)
	#local Normal=vcross(Second-First,Third-First);
	#local Distance=vdot(First,Normal)/vlength(Normal);
	plane {
		Normal,
		Distance
		pigment { Col_Glass_Ruby }
	}
#end

// Make a bead to mark a point
// These are helper points so we give a glass finish
#macro Point(Place)
	sphere {
		Place,
		(1.5)*Thick
		pigment { Col_Vicksbottle }
	}
#end

// Make a line joining the first and second point which is
// symmetrically extended on both sides and has specified length. The
// line is represented by a cylinder around it of specified radius
#macro Line(First,Second,Radius) 
	#local Distance = vlength(First-Second);
	#local Centre = (First+Second)/2;
	#local Vector = (First-Second)/Distance;
	cylinder {
		Centre-(Length/2)*Vector,
		Centre+(Length/2)*Vector,
		Radius
	}
#end

//// Unused!
// Make a line joining the first and second point which is
// symmetrically extended on both sides and has specified length. The
// line is represented by a cylinder around it of specified radius
// In addition the line slides through the picture
#macro SlideLine(First, Second, Radius, Begin, End) 
	#local Distance = vlength(First-Second);
	#local Centre = (First+Second)/2;
	#local Vector = (First-Second)/Distance;
	object {
	#if (clock > Begin)
		#local Myclock = max(0,(End - clock)/(End-Begin));
		cylinder {
			Centre-(Length/2)*Vector,
			Centre+(Length/2)*Vector,
			Radius
		}
		translate 6*Myclock*Length*Vector
	#end
	}
#end

// Make wooden stick
#macro Wooden(First,Second)
	object {
		Line(First,Second,Thick)
		texture { T_Wood19 }
	}
#end

// Make a "needle"
#macro Needle(First,Second)
	object {
		Line(First,Second,Thin)
		pigment { color Silver }
	}
#end

// Make a helper line
#macro Helper(First,Second)
	object {
		Line(First,Second,VThin)
		pigment { color Col_Emerald_03 }
	}
#end


// Lights!
global_settings { ambient_light rgb<0.9,0.9,1.0> }

light_source { <-1, -1, 15> color White shadowless }
light_source { <-10, 0, 4> color 0.5*White shadowless }
light_source { <10, 0, 4> color 0.5*White shadowless }

light_source { <0, -13, 4> color 0.5*White shadowless }
light_source { <0, 13, 4> color 0.5*White shadowless }

background { color rgb <0.5, 0.6, 0.7> }

// Camera!
camera {
//// if we want a "geometric"
//// rather than perspective view
//  orthographic
  location <0, 0, 13>
  look_at  <0, 0,  0>
}

///////////////////
// This section creates the picture
///////////////////

// The point corresponding to the given parameter
// which lies on the line joining the first and second points.
#macro Interpolate(First,Second,Lam)
	Lam*First+(1-Lam)*Second
#end

// The point of intersection of the line joining A and B with the
// plane determined by U, V, and W.
#macro Intersection(A,B,U,V,W)
	#local Normal=vcross(V-U,W-U);
	#local Lam=vdot(U-B,Normal)/vdot(A-B,Normal);
	Interpolate(A,B,Lam)
#end

// Action!
// The initial 6 lines in the quadric
#declare ac=3/5;
#declare as=4/5;
#declare x_16=<1.0,0,0>;
#declare x_15=<1.0,(1+ac)/as,(1+ac)/as>;
#declare x_14=<1.0,-(1+ac)/as,-(1+ac)/as>;

#declare x_25=<-ac,as,0>;
#declare x_24=<-1/ac,0.0,as/ac>;
#declare x_26=<1.0,(1+ac)/as,-(1+ac)/as>;

#declare x_34=<-ac,-as,0>;
#declare x_36=<1.0,-(1+ac)/as,(1+ac)/as>;
#declare x_35=<-1/ac,0.0,-as/ac>;

// These three points define the plane containing R_16, R_25, R_34.
#declare pt1=<0,0,1>;
#declare pt2=<1,0,1>;
#declare pt3=<0,1,1>;

#declare y_16=Intersection(x_15,x_16,pt1,pt2,pt3);
#declare z_16=Intersection(x_16,x_26,pt1,pt2,pt3);
#declare y_25=Intersection(x_25,x_26,pt1,pt2,pt3);
// The remaining points.
#declare z_25 = Intersection(x_15,x_25,y_16,z_16,y_25);
#declare y_34 = Intersection(x_35,x_36,y_16,z_16,z_25);
#declare z_34 = Intersection(x_14,x_24,y_16,z_16,z_25);

// Now we pick the point on P_2 that is needed to pick Q_1
#declare b=-1.7;
#declare x_21=b*x_25+(1-b)*x_26;
// Q_1 is constructed using the fact the it meets
// R_16 P_2 and P_3. So this is the point of intersection 
// of the span of R_16 and x_21 with the line P_3
#declare x_31 = Intersection(x_35,x_36,x_21,y_16,z_16);

// Now we have P_1, P_2, P_3, Q_1, Q_4, Q_5, Q_6

// P_6 is the line that lies the plane created by R_16 and Q_1
// and it meets Q_4 and Q_5.
#declare x_64=Intersection(x_14,x_24,y_16,z_16,x_21);
#declare x_65=Intersection(x_15,x_25,y_16,z_16,x_21);
#declare x_61=Intersection(x_21,x_31,x_16,x_64,x_65);

// R_46 is the line that lies the plane created by P_6 and Q_4
// and it meets R_25 and Q_6.
#declare z_46=Intersection(x_16,x_26,x_65,x_14,x_24);
#declare w_2546=Intersection(y_25,z_25,x_65,x_14,x_24);

// P_4 is the line that lies the plane created by R_46 and Q_6
// and it meets Q_1 and Q_5.
#declare x_41=Intersection(x_21,x_31,w_2546,x_16,x_26);
#declare x_45=Intersection(x_15,x_25,w_2546,x_16,x_26);
#declare x_46=Intersection(x_16,x_26,x_35,x_41,x_45);

// R_56 is the line that lies the plane created by P_6 and Q_5
// and it meets R_34 and Q_6.
#declare z_56=Intersection(x_16,x_26,x_64,x_15,x_25);
#declare w_3456=Intersection(y_34,z_34,x_64,x_15,x_25);

// P_5 is the line that lies the plane created by R_56 and Q_6
// and it meets Q_1 and Q_4.
#declare x_51=Intersection(x_21,x_31,w_3456,x_16,x_26);
#declare x_54=Intersection(x_14,x_24,w_3456,x_16,x_26);
#declare x_56=Intersection(x_16,x_26,x_35,x_51,x_54);

// Q_2 is the line that lies the plane created by R_25 and P_5
// and it meets P_1 and P_3.
#declare x_12=Intersection(x_15,x_16,y_25,x_51,x_54);
#declare x_32=Intersection(x_35,x_36,y_25,x_51,x_54);
#declare x_42=Intersection(x_41,x_45,x_26,x_12,x_32);
#declare x_52=Intersection(x_51,x_54,x_26,x_12,x_32);
#declare x_62=Intersection(x_61,x_65,x_26,x_12,x_32);

// Q_3 is the line that lies the plane created by R_34 and P_4
// and it meets P_1 and P_2.
#declare x_13=Intersection(x_15,x_16,y_34,x_41,x_45);
#declare x_23=Intersection(x_25,x_26,y_34,x_41,x_45);
#declare x_43=Intersection(x_41,x_45,x_26,x_13,x_23);
#declare x_53=Intersection(x_51,x_54,x_26,x_13,x_23);
#declare x_63=Intersection(x_61,x_65,x_26,x_13,x_23);

// All the R_ij's
// Of these R_16, R_25, R_34, R_46, R_56 have been constructed
// earlier.

// R_12 is the line that lies the plane created by P_1 and Q_2
// and it meets Q_1 and P_2.
#declare y_21=Intersection(x_25,x_26,x_15,x_16,x_32);
#declare z_21=Intersection(x_21,x_31,x_15,x_16,x_32);

// R_13 is the line that lies the plane created by P_1 and Q_3
// and it meets Q_1 and P_3.
#declare y_31=Intersection(x_35,x_36,x_15,x_16,x_23);
#declare z_31=Intersection(x_21,x_31,x_15,x_16,x_23);

// R_14 is the line that lies the plane created by P_1 and Q_4
// and it meets Q_1 and P_4.
#declare y_41=Intersection(x_41,x_45,x_15,x_16,x_34);
#declare z_41=Intersection(x_21,x_31,x_15,x_16,x_34);

// R_15 is the line that lies the plane created by P_1 and Q_5
// and it meets Q_1 and P_5.
#declare y_51=Intersection(x_51,x_54,x_15,x_16,x_25);
#declare z_51=Intersection(x_21,x_31,x_15,x_16,x_25);

// R_23 is the line that lies the plane created by P_2 and Q_3
// and it meets Q_2 and P_3.
#declare y_32=Intersection(x_35,x_36,x_25,x_26,x_13);
#declare z_32=Intersection(x_12,x_32,x_25,x_26,x_13);

// R_24 is the line that lies the plane created by P_2 and Q_4
// and it meets Q_2 and P_4.
#declare y_42=Intersection(x_41,x_45,x_25,x_26,x_34);
#declare z_42=Intersection(x_12,x_32,x_25,x_26,x_34);

// R_26 is the line that lies the plane created by P_2 and Q_6
// and it meets Q_2 and P_6.
#declare y_62=Intersection(x_64,x_65,x_25,x_26,x_16);
#declare z_62=Intersection(x_12,x_32,x_25,x_26,x_16);

// R_35 is the line that lies the plane created by P_3 and Q_5
// and it meets Q_3 and P_5.
#declare y_53=Intersection(x_51,x_54,x_15,x_31,x_25);
#declare z_53=Intersection(x_23,x_13,x_15,x_31,x_25);

// R_36 is the line that lies the plane created by P_3 and Q_6
// and it meets Q_3 and P_6.
#declare y_63=Intersection(x_64,x_65,x_31,x_26,x_16);
#declare z_63=Intersection(x_13,x_23,x_31,x_26,x_16);

// R_45 is the line that lies the plane created by P_4 and Q_5
// and it meets Q_4 and P_5.
#declare y_54=Intersection(x_51,x_54,x_15,x_41,x_25);
#declare z_54=Intersection(x_41,x_45,x_15,x_41,x_25);

#declare P_1 = Wooden(x_15,x_16);
#declare P_2 = Wooden(x_25,x_26);
#declare P_3 = Wooden(x_35,x_36);
#declare P_4 = Wooden(x_45,x_41);
#declare P_5 = Wooden(x_54,x_51);
#declare P_6 = Wooden(x_65,x_64);

#declare Q_1 = Needle(x_21,x_31);
#declare Q_2 = Needle(x_12,x_32);
#declare Q_3 = Needle(x_13,x_23);
#declare Q_4 = Needle(x_24,x_14);
#declare Q_5 = Needle(x_15,x_25);
#declare Q_6 = Needle(x_16,x_26);

//#declare Q_1 = SlideLine(x_21,x_31,0.12);
//#declare Q_2 = SlideLine(x_12,x_32,0.12);
//#declare Q_3 = SlideLine(x_13,x_23,0.24);
//#declare Q_4 = SlideLine(x_24,x_14,0.36);
//#declare Q_5 = SlideLine(x_15,x_25,0.48);
//#declare Q_6 = SlideLine(x_16,x_26,0.60);

#declare R_16 = Helper(y_16,z_16);
#declare R_25 = Helper(y_25,z_25);
#declare R_34 = Helper(y_34,z_34);
#declare R_46 = Helper(w_2546,z_46);
#declare R_56 = Helper(w_3456,z_56);

#declare R_12 = Helper(y_21,z_21);
#declare R_13 = Helper(y_31,z_31);
#declare R_14 = Helper(y_41,z_41);
#declare R_15 = Helper(y_51,z_51);
#declare R_23 = Helper(y_32,z_32);
#declare R_24 = Helper(y_42,z_42);
#declare R_26 = Helper(y_62,z_62);
#declare R_35 = Helper(y_53,z_53);
#declare R_36 = Helper(y_63,z_63);
#declare R_45 = Helper(y_54,z_54);

#declare Pict = union {
// First collection of lines is the
// collection P_1, P_2, P_3, Q_5, Q_6.
	object { P_1 }
#if (clock >= 0.2)
	object { P_2 }
#end
#if (clock >= 0.4 & clock < 0.7)
	union {
		object { Point(x_15) }
		object { Point(x_25) }
	}
#end
#if (clock >= 0.5)
	object { Q_5 }
#end
#if (clock >= 0.7 & clock < 1.1)
	union {
		object { Point(x_16) }
		object { Point(x_26) }
	}
#end
#if (clock >= 1.0)
	object { Q_6 }
#end
#if (clock >= 1.2 & clock < 1.6)
	union {
		object { Point(x_35) }
		object { Point(x_36) }
	}
#end
#if (clock >= 1.5)
 	object { P_3 }
#end

// Construction of Q_4
#if (clock >= 1.6 & clock < 2.6)
	object { Point(x_34) }
#end
#if (clock >= 1.8 & clock < 2.1)
	object { Plane(x_34,x_16,x_15) }
#end
#if (clock >= 2.0 & clock < 2.6)
	object { Point(x_24) }
#end
#if (clock >= 2.3)
	object { Q_4 }
#end
#if (clock >= 2.4 & clock < 2.6)
	object { Point(x_14) }
#end
// End of construction of Q_4

// Construction of the three helper lines
// R_16, R_25, R_34 which all lie in a plane
#if (clock >= 3.0 & clock < 4.6) 
	object { Plane(pt1,pt2,pt3) }
#end
#if (clock >= 3.2 & clock < 3.6) 
	union {
		object { Point(y_16) }
		object { Point(z_16) }
	}
#end
#if (clock >= 3.5 & clock < 7) 
	object { R_16 }
#end
#if (clock >= 3.7 & clock < 4.2) 
	union {
		object { Point(y_25) }
		object { Point(z_25) }
	}
#end
#if (clock >= 4.0 & clock < 9.8) 
	object { R_25 }
#end
#if (clock >= 4.3 & clock < 4.7) 
	union {
		object { Point(y_34) }
		object { Point(z_34) }
	}
#end
#if (clock >= 4.5 & clock < 10.5) 
	object { R_34 }
#end
// End of construction of the three helper lines

// We now construct the remaining lines
// To construct these we first need to pick x_21 on P_2
#if (clock >= 4.7 & clock < 5.8)
	object { Point(x_21) }
#end
// Take the plane containing x_21 and R_16
// It meets Q_1 at the point where Q_1 meets P_3
// It contains P_6
#if (clock >= 5.0 & clock < 5.4)
	object { Plane(x_21,y_16,z_16) }
#end
#if (clock >= 5.2 & clock < 5.8)
	object { Point(x_31) }
#end
#if (clock >= 5.2 & clock < 6.3)
	union {
		object { Point(x_64) }
		object { Point(x_65) }
	}
#end
#if (clock >= 5.7)
	object { Q_1 }
#end
#if (clock >= 6.0)
	object { P_6 }
#end
#if (clock >= 6.1 & clock < 6.3)
	object { Point(x_61) }
#end
// Finished constructing Q_1 & P_6

// Construct \pi_46
#if (clock >= 6.5 & clock < 6.8)
	object { Plane(x_65,x_14,x_24) }
#end
#if (clock >= 6.7 & clock < 7.2)
	object { Point(w_2546) }
#end
#if (clock >= 7.0 & clock < 7.6) 
	object { Plane(w_2546,x_16,x_26) }
#end
// End of construction of \pi_46

// Construct P_4
#if (clock >= 7.5 & clock < 7.9)
	union {
		object { Point(x_41) }
		object { Point(x_45) }
	}
#end
#if (clock >= 7.7)
	object { P_4 }
#end
#if (clock >= 7.8 & clock < 7.9)
	object { Point(x_46) }
#end
// End of P_4 construction

// Construct \pi_56
#if (clock >= 8.0 & clock < 8.3)
	object { Plane(x_64,x_15,x_25) }
#end
#if (clock >= 8.2 & clock < 8.6)
	object { Point(w_3456) }
#end
#if (clock >= 8.5 & clock < 8.8) 
	object { Plane(w_3456,x_16,x_26) }
#end
// End of construction of \pi_56

// Construct P_5
#if (clock >= 8.7 & clock < 9.2)
	union {
		object { Point(x_51) }
		object { Point(x_54) }
	}
#end
#if (clock >= 9.0)
	object { P_5 }
#end
#if (clock >= 9.1 & clock < 9.2)
	object { Point(x_56) }
#end
// End of P_5 construction

// Construction of Q_2
#if (clock >= 9.5 & clock < 9.8)
	object { Plane(y_25,x_51,x_54) }
#end
#if (clock >= 9.7 & clock < 10.2)
	union {
		object { Point(x_12) }
		object { Point(x_32) }
	}
#end
#if (clock >= 9.9)
	object { Q_2 }
#end
#if (clock >= 10.0 & clock < 10.2)
	union {
		object { Point(x_42) }
		object { Point(x_52) }
		object { Point(x_62) }
	}
#end
// End of Construction of Q_2

// Construction of Q_3
#if (clock >= 10.3 & clock < 10.5)
	object { Plane(y_34,x_41,x_45) }
#end
#if (clock >= 10.4 & clock < 10.8)
	union {
		object { Point(x_13) }
		object { Point(x_23) }
	}
#end
#if (clock >= 10.5)
	object { Q_3 pigment { color Silver }}
#end
#if (clock >= 10.6 & clock < 10.8)
	union {
		object { Point(x_43) }
		object { Point(x_53) }
		object { Point(x_63) }
	}
#end
}

object {
	Pict
	rotate <0,-15,0>
//	rotate <20,0,0>
}


